""" grab-bag file """


##main application version
__version__ = "0.26"

import time

#creates a separate db in $HOME/dowserdev, etc. Intended for heavy development.
DEV = False


## embryo datatype for lists of search results.
class ResultList(list):
	def __init__(self):
		self.results = []

## default preferences and settings.
## nb: the 'VERSION' tag here is the serial number of the prefs array, not
## connected to the program's version.
prefdefault = {
	'VERSION':17,
	'port':5309,
	'lang':'en', 
	'max_page_size':4096000, 
	'crawler_sleep':4.00, 
	'crawler_delay':37.00, 
	'crawler_retry':3,
	'crawler_revisit_after':15, 
	'metasearch_timeout':2.250, 
	'max_page':30,
	'disk_quota':1073741824,
	'site_searches':['wikipedia.org', 'sourceforge.net'],
	'default_engine':'web',
	'allow_remote_clients':False,
	'debug':False,
	'proxy':'proxy.example.com',
	'proxy_port':1234,
	'use_proxy':False,
	'check_upgrades':True,
	'launch_browser':True,
	'last_upgrade_check':time.time(),
	'last_good_check': time.time(),
	'latest_version':float(__version__),
	'debug': False,
	'filters':{
		'no shopping': {'on':True, 'flags':'-shopping -prices -cart'},
		'no blogs': {'on':True, 'flags':'-blog -trackback'}
	 },
	 'show_filters':True
}

prefs = {} ## current preferences
header = idx_header = footer = idx_footer = home = preffile = None


DEFAULT_MIMETYPE = "text/html"

##################################################################
import os
import urllib
import string
import re
import cPickle
import time
import fpformat

def debug(msg):
	'yeah, yeah, I know. simplistic.'
	global prefs
	if prefs['debug']:
		print msg

def dedupe(arr):
	'deduplicates and sorts an array or list'
	tmp = {}
	# Arr! Avast, ye procedural dogs! Prepare to be comprehended!
	[tmp.update({r:None}) for r in arr]
	tmp = tmp.keys()
	tmp.sort()
	return tmp

##todo: is there a case where failing to make a directory is fatal?
def mkdir(d):
	try:
		os.makedirs(d)
	except:
		pass

r_whitespace = re.compile(r"[\n\s\t\v\r]+", re.M)
def stripper(text):
	'strips tailing, leading and repeating whitespace. removes newlines.'
	global r_whitespace
	text = r_whitespace.sub(" ", text)
	text = string.strip(text)
	return text

import sgmllib

class Stripper(sgmllib.SGMLParser):
        def __init__(self):
                sgmllib.SGMLParser.__init__(self)
                
        def strip(self, some_html):
                self.theString = ""
                self.feed(some_html)
                self.close()
                return self.theString
                
        def handle_data(self, data):
                self.theString += data





## todo: this does not handle malformed stuff, or
## text inside scripts, iframes, styles, objects, etc.
r_html = re.compile(r"</?[^>]+>")
r_style = re.compile(r"</?style[^>]*>")
def striphtml(text):
	#global r_html, r_style
	#return r_html.sub(" ", text)
	stripper = Stripper()
	return stripper.strip(text)

def shortdate(unixtime=None):
	'returns given time or current time in "day month, year" format'
	if not unixtime: unixtime = time.time()
	return time.strftime("%d %b, %Y", time.localtime(unixtime))
	
def longdate(unixtime=None):
	'returns given time or current time in "day month, year time" format'
	if not unixtime: unixtime = time.time()
	return time.strftime("%d %b, %Y %I:%M %p", time.localtime(unixtime))

"""string formats for K, M, G, T, etc"""
def shortbytes(size):
	unit = "KB"
	s = size / 1024
	
	if s < 1:
		s = size
		unit = "bytes" ##todo: i18n
	
	if s >= 1000:
		s = size / 1048576
		unit = "MB"
	if s >= 1000:
		s = size / 1073741824
		unit = "GB"
	if s>= 1000:
		s = size / 1099511627776
		unit = "TB"
	if s>= 1000:
		s = size / 1125899906842624
		unit = "PetaBytes (!!)"
		
	s = fpformat.fix(s, 1)
	if s[-1] == "0": #trim off ".0"
		s = s[0:-2] 
	
	return "%s %s" % (s, unit)

##todo: as-is this will not use the system-configured proxy
def getproxy():
	global prefs
	prox = {}
	if prefs['use_proxy']:
		prox = {'http': "%s:%d" % (prefs['proxy'], prefs['proxy_port'])}
	return prox

"""
Just replaces quotes and < >
"""
def htmlencode(data):
	data = string.replace(data, "\"", "&quot;")
	data = string.replace(data, "<", "&lt;")
	data = string.replace(data, ">", "&gt;")
	return data

## doubles up single-quotes. 
## todo: should be in db_<database>.py
def sqlquote(text):
	return string.replace(text,"'", "''")	

"""
takes the QUERY_STRING environment variable and returns the key/value pairs
as a dictionary
"""
def parseargs(env=None):
	if env:
		if not env.has_key('QUERY_STRING'): 
			return []
		qs = env['QUERY_STRING']
	else:
		qs = os.environ['QUERY_STRING']
		
	qs = urllib.unquote_plus(qs)
	qargs = {'--qs': qs}
	pairs = string.split(qs, "&")
	for p in pairs:
	
		kv = string.split(p, "=", 1)
		if len(kv) > 1:
			key = kv[0]
			value = ''.join(kv[1:])
			if not qargs.has_key(key):
				qargs[key] = urllib.unquote_plus(value)
			else:
				if isinstance(qargs[key], list):
					qargs[key] += [urllib.unquote_plus(value)]
				else:
					qargs[key] = [qargs[key], urllib.unquote_plus(value)]

	return qargs



""" find the user's home path. defaults to the CWD"""
def gethome():
	global home, DEV
	if home: return home
	
	o = os.environ
	tags = ['HOME', 'HOMEPATH']
	for t in tags:
		if o.has_key(t) and o[t] and string.strip(o[t]) != "": 
			home = o[t]
	
	if not home or string.strip(home) == "": home = '.'
	
	## debug for dev versions.
	if DEV:
		home += "/dowserdev"
		
	print home
		
	return home


##makes sure the new input number is an int, and within range. If not,
## returns the 'orig' value.
## todo: will float instead of int fuck this? port 3.14159? :)
def numer(num, orig, minn=None, maxx=None):
	try:
		num = int(num)		
		if (minn and maxx) and (num < minn) or (num > maxx):
			raise
	except:
		num = orig
		
	return num



##saves an array to the prefs file
def saveprefs(newprefs=None):
	if not newprefs:
		global prefs
		newprefs = prefs
	f = open(preffile, 'w')
	p = cPickle.Pickler(f)
	p.dump(newprefs)
	f.close()
	init()
	import msg
	msg.init()



##writes default prefs to disk, returns default values
def initdefault():
	saveprefs(prefdefault)
	debug( "loading default prefs...." )
	return prefdefault


#returns the code for a single html select box
def mkselect(options, name, selected=None):
	r = []	
	for opt in options:
	
		##mark the "selected" option
		checked = ''
		if opt[0] == selected:
			checked = """selected="selected" """
			
		r.append("""<option %s value="%s">%s</option>""" % (checked, opt[0], opt[1]))
		
	return ("""<select name="%s">\n""" % str(name)) + ("\n".join(r)) + "\n</select>"
	


def init():
	global DEV

	##home
	global home, preffile
	home = gethome()
	mkdir(home + "/dowser")
	preffile = home + '/dowser/prefs.pickle'



	##preferences
	global prefs
	p = None
	try: 
		f = open(preffile, 'r')
		p = cPickle.Unpickler(f)
		p = p.load()
		f.close()
		
		##check for version mismatch and update.
		##newer versions MUST be backwards compatible; this may
		##leave junk around in the pickle.
		if p['VERSION'] < prefdefault['VERSION']:
			newver = prefdefault['VERSION']
			p2 = prefdefault
			p2.update(p)
			p2['VERSION'] = newver
			p = p2
			saveprefs(p)
			
	except:
		prefs = initdefault()
	
	
	if not p:
		prefs = initdefault()
	else:
		prefs = p
		
	if DEV:
		prefs['port'] = 6666
		
		

	#version link
	get_latest = ""
	if float(__version__) < prefs['latest_version']:
		get_latest = """&nbsp;&nbsp;&#0149;&nbsp;&nbsp;<b><a 
		href="http://sourceforge.net/project/showfiles.php?group_id=104361">%(LATEST)s 
		""" + str(prefs['latest_version']) + """</a></b>"""
		
	## search sites list
	opts = (('web', '%(WEB)s'), ('web:google','&nbsp;&#0149;&nbsp;%(GOOGLE)s'), ('web:teoma', '&nbsp;&#0149;&nbsp;%(TEOMA)s'), ('local','%(LOCAL_CACHE)s'))
	for s in prefs['site_searches']:
		opts = opts + (('site:'+s,s),)
	search_select = mkselect(opts, 's', prefs['default_engine'])


	## filters list
	filters = []
	for name, f in prefs['filters'].iteritems():
		checked = ""
		if f['on']: checked=""" checked="checked" """
		filters.append("""<input type="checkbox" name="filters" 
			value="%s" %s /> %s &nbsp;""" % (name, checked, name))
	
	if prefs['show_filters']:
		style=""
		checked=""" checked="checked" """
	else:
		style = """style="display:none;" """
		checked=""
		
	search_filters = """<!--
		&nbsp;&nbsp;&nbsp;&nbsp;
		<small><small><input type="checkbox" onclick="show_filters();" 
		""" +checked+"""/> %(SHOW_FILTERS)s
		-->
		<div id="filters" 
		""" + style +"""><small><small>
		""" + '\n'.join(filters) + """&nbsp;&nbsp;
			<i><a href="/filters">%(EDIT_FILTERS)s</a></i>
			</small></small></div>
		""" 

	###normal page header##############

	"""
<a href=/index.html><img src=/media/d.png align=absmiddle border=0 
		alt = "Dowser version """ +fpformat.fix(__version__, 2)+""" " /></a> 
	"""

	global header
	header = """<html>
<head>
	<meta http-equiv="content-type" content="text/html; charset=utf-8" />
	<title>%(title)s</title>
	<link rel="stylesheet" href="/media/dowser.css" type="text/css" />
	<script src="/media/dowser.js"></script>
</head>	
<body>
	<div id=bigdiv>
	
	
		<table class=logo width="100%%" border=0 cellpadding=0 cellspacing=0><td>

		<div class=menu>
			<a href="/browse?by=kw">%(LOCAL_CACHE)s</a>
			
			&nbsp;&nbsp;&#0149;&nbsp;&nbsp;
			<a href="/browse?by=date">%(LATEST_CACHE)s</a>
			
			&nbsp;&nbsp;&#0149;&nbsp;&nbsp;
			<a href="/prefs">%(PREFS)s</a>
			
			
			&nbsp;&nbsp;&#0149;&nbsp;&nbsp;
			<a href="http://dowser.sourceforge.net">%(ABOUT)s</a>
			
			""" + get_latest + """
		</div>
		</td><td align="right">			&nbsp;
			<a href="/"><big class="logo">dowser</big></a><small><small><small> """ + str(__version__) + """</small></small></small>
		</td></table> 
		
		
		<div class=frm>
		
			<form action=/search name="frm">
				<table border=0 cellpadding=0 cellspacing=0><td>
&nbsp;				</td><td>
				<input type=text size=75 name=q class=q id=q value="%(q)s" >
				<input type=submit class=sb value="%(SEARCH)s">
				

""" + search_select + """
				<!-- search filters -->
""" + search_filters + """
				</td></table> 
			</form>
		</div>
"""
	if DEV:
		header="DEBUG" + header


	###############footer######################
	global footer
	footer = """	
</div>
</ul></ol>
	</td></table>

</body>
</html>
"""
	##########index#############################
	global idx_footer
	idx_footer = """	
</div>
</ul></ol>

</body></html>"""

	global idx_header
	idx_header = """<html>
<head>
	<meta http-equiv="content-type" content="text/html; charset=utf-8" />
	<title>%(title)s</title>
	<link rel=stylesheet href=/media/dowser.css type=text/css />
	<script src=/media/dowser.js></script>
</head>	
<body onload="faux_cuss()">

	<table width="100%%" height="100%%"><td align="center">
	<div id=idx_bigdiv>
	
	
		<table class=logo border=0 cellpadding=0 cellspacing=0 width="600"><td>
		
		<div class=menu>
			<a href="/browse?by=kw">%(LOCAL_CACHE)s</a>
			
			&nbsp;&nbsp;&#0149;&nbsp;&nbsp;
			<a href="/browse?by=date">%(LATEST_CACHE)s</a>
			
			&nbsp;&nbsp;&#0149;&nbsp;&nbsp;
			<a href="/prefs">%(PREFS)s</a>
			
			
			&nbsp;&nbsp;&#0149;&nbsp;&nbsp;
			<a href="http://dowser.sourceforge.net">%(ABOUT)s</a>
			
			""" + get_latest + """
			

		</div>

		
		</td><td align="right">
		&nbsp;
			<a href="/"><big class="logo">dowser</big></a><small><small><small> """ + str(__version__) + """</small></small></small>
			
		</td></table> 
		
		
		<div class=frm width="600" align="left">
		
			<form action=/search name="frm">
				<table border=0 cellpadding=0 cellspacing=0>
				<td>
				</td><td>
				<input type=text size=50 name=q class=q id=q value="%(q)s" >
				<input type=submit class=sb value="%(SEARCH)s">

""" + search_select + """ 
				<!-- search filters -->
""" + search_filters + """
				</td></table> 
			</form>
		</div>
"""

	if DEV:
		idx_header="DEBUG" + idx_header

####### run it once.
init()

prefs['debug'] = False
##prefs['last_upgrade_check'] = time.time() - (86400 * 8)